@@ -11,14 +11,22 @@ import yaml
 
 from flex.context_managers import ErrorCollection
 from flex.exceptions import ValidationError
-from flex.serializers.core import (
-    SwaggerSerializer,
-    SchemaSerializer,
+from flex.loading.definitions import (
+    definitions_validator,
 )
-from flex.serializers.definitions import SwaggerDefinitionsSerializer
-from flex.utils import prettify_errors
-from flex.validation.request import generate_request_validator
-from flex.validation.response import generate_response_validator
+from flex.loading.schema import (
+    swagger_schema_validator,
+)
+from flex.loading.schema.paths.path_item.operation.responses.single.schema import (
+    schema_validator,
+)
+from flex.http import (
+    normalize_request,
+    normalize_response,
+)
+from flex.validation.common import validate_object
+from flex.validation.request import validate_request
+from flex.validation.response import validate_response
 
 
 def load_source(source):
@@ -70,28 +78,16 @@ def load_source(source):
 
 
 def parse(raw_schema):
-    definitions_serializer = SwaggerDefinitionsSerializer(
-        data=raw_schema,
-    )
-    if not definitions_serializer.is_valid():
-        message = "Swagger definitions did not validate:\n\n"
-        message += prettify_errors(definitions_serializer.errors)
-        raise ValueError(message)
+    context = {
+        'deferred_references': set(),
+    }
+    swagger_definitions = definitions_validator(raw_schema, context=context)
 
-    swagger_definitions = definitions_serializer.save()
-
-    swagger_serializer = SwaggerSerializer(
-        swagger_definitions,
-        data=raw_schema,
+    swagger_schema = swagger_schema_validator(
+        raw_schema,
         context=swagger_definitions,
     )
-
-    if not swagger_serializer.is_valid():
-        message = "Swagger schema did not validate:\n\n"
-        message += prettify_errors(swagger_serializer.errors)
-        raise ValueError(message)
-
-    return swagger_serializer.save()
+    return swagger_schema
 
 
 def load(target):
@@ -103,32 +99,51 @@ def load(target):
     return parse(raw_schema)
 
 
-def validate(schema, target=None, **kwargs):
+def validate(raw_schema, target=None, **kwargs):
     """
     Given the python representation of a JSONschema as defined in the swagger
     spec, validate that the schema complies to spec.  If `target` is provided,
     that target will be validated against the provided schema.
     """
-    schema_serializer = SchemaSerializer(data=schema, **kwargs)
-    if not schema_serializer.is_valid():
-        message = "JSON Schema did not validate:\n\n"
-        message += prettify_errors(schema_serializer.errors)
-        raise ValidationError(message)
-
+    schema = schema_validator(raw_schema, **kwargs)
     if target is not None:
-        validator = schema_serializer.save()
-        validator(target)
+        validate_object(target, schema=schema, **kwargs)
 
 
-def validate_api_call(schema, request, response):
+def validate_api_request(schema, raw_request):
+    request = normalize_request(raw_request)
+
+    with ErrorCollection():
+        validate_request(request=request, schema=schema)
+
+
+def validate_api_call(schema, raw_request, raw_response):
+    """
+    Validate the request/response cycle of an api call against a swagger
+    schema.  Request/Response objects from the `requests` and `urllib` library
+    are supported.
+    """
+    request = normalize_request(raw_request)
+    response = None
+    if raw_response is not None:
+        response = normalize_response(raw_response)
+
     with ErrorCollection() as errors:
         try:
-            operation_definition = generate_request_validator(schema)(request)
+            validate_request(
+                request=request,
+                schema=schema,
+            )
         except ValidationError as err:
             errors['request'].add_error(err.messages or getattr(err, 'detail'))
             return
 
         try:
-            generate_response_validator(operation_definition, schema)(response)
+            if response is not None:
+                validate_response(
+                    response=response,
+                    request_method=request.method,
+                    schema=schema,
+                )
         except ValidationError as err:
             errors['response'].add_error(err.messages or getattr(err, 'detail'))
