@@ -1,5 +1,6 @@
 import six
 
+import io
 import http
 import urllib
 
@@ -8,12 +9,27 @@ import json
 
 from flex.constants import EMPTY
 
-_tornado_available = True
 try:
     import tornado.httpclient
     import tornado.httpserver
 except ImportError:
     _tornado_available = False
+else:
+    _tornado_available = True
+
+try:
+    import falcon  # noqa
+except ImportError:
+    _falcon_available = False
+else:
+    _falcon_available = True
+
+try:
+    import webob
+except ImportError:
+    _webob_available = False
+else:
+    _webob_available = True
 
 
 class URLMixin(object):
@@ -58,8 +74,13 @@ class Request(URLMixin):
             return self.body
         elif self.body is EMPTY:
             return EMPTY
-        elif self.content_type == 'application/json':
-            return json.loads(self.body)
+        elif self.content_type.startswith('application/json'):
+            if type(self.body) == bytes:
+                return json.loads(self.body.decode('utf-8'))
+            else:
+                return json.loads(self.body)
+        elif self.content_type == 'application/x-www-form-urlencoded':
+            return dict(urlparse.parse_qsl(self.body))
         else:
             raise NotImplementedError("No parser for content type")
 
@@ -148,11 +169,45 @@ def _normalize_tornado_request(request):
     )
 
 
+def _normalize_falcon_request(request):
+    if not _falcon_available:
+        raise TypeError("Falcon is not installed")
+
+    # Falcon request.stream has to be replaced because it can be read only once
+    request.stream = io.BytesIO(request.stream.read())
+
+    return Request(
+        url=request.url,
+        body=request.stream.getvalue().decode(),
+        method=request.method.lower(),
+        content_type=request.content_type,
+        request=request,
+    )
+
+
+def _normalize_webob_request(request):
+    if not _webob_available:
+        raise TypeError("webob is not installed")
+
+    if not isinstance(request, webob.BaseRequest):
+            raise TypeError("Cannot normalize this request object")
+
+    return Request(
+        url=request.url,
+        body=request.body,
+        method=request.method.lower(),
+        content_type=request.content_type,
+        request=request,
+    )
+
+
 REQUEST_NORMALIZERS = (
     _normalize_python2_urllib_request,
     _normalize_python3_urllib_request,
     _normalize_requests_request,
+    _normalize_webob_request,
     _normalize_tornado_request,
+    _normalize_falcon_request,
 )
 
 
@@ -186,7 +241,7 @@ class Response(URLMixin):
         self.request = request
         self.content = content
         self.url = url
-        self.status_code = status_code
+        self.status_code = str(status_code)
         self.content_type = content_type
         self.headers = headers or {}
 
@@ -198,7 +253,7 @@ class Response(URLMixin):
     def data(self):
         if self.content is EMPTY:
             return self.content
-        elif self.content_type == 'application/json':
+        elif self.content_type.startswith('application/json'):
             if isinstance(self.content, six.binary_type):
                 return json.loads(six.text_type(self.content, encoding='utf-8'))
             else:
@@ -264,9 +319,36 @@ def _normalize_tornado_response(response, request=None):
     )
 
 
+def _normalize_webob_response(response, request=None):
+    if not _webob_available:
+        raise TypeError("webob is not installed")
+
+    if not isinstance(response, webob.Response):
+        raise TypeError("Cannot normalize this response object")
+
+    url = None
+
+    if request:
+        url = request.url
+    elif response.request:
+        url = response.request.url
+    else:
+        raise TypeError("Normalized webob object needs a path")
+
+    return Response(
+        request=request,
+        content=response.body,
+        url=url,
+        status_code=response.status.split()[0],
+        content_type=response.content_type,
+        response=response,
+    )
+
+
 RESPONSE_NORMALIZERS = (
     _normalize_urllib_response,
     _normalize_requests_response,
+    _normalize_webob_response,
     _normalize_tornado_response,
 )
 
