@@ -9,6 +9,7 @@ import json
 
 import six
 
+
 from flex.exceptions import (
     ValidationError,
     ErrorDict,
@@ -32,6 +33,7 @@ from flex.paths import (
     match_path_to_api_path,
 )
 from flex.constants import (
+    NULL,
     NUMBER,
     STRING,
     ARRAY,
@@ -67,6 +69,10 @@ def generate_type_validator(type_, **kwargs):
         types = type_
     else:
         types = (type_,)
+    # support x-nullable since Swagger 2.0 doesn't support null type
+    # (see https://github.com/OAI/OpenAPI-Specification/issues/229)
+    if kwargs.get('x-nullable', False) and NULL not in types:
+        types = types + (NULL,)
     return functools.partial(validate_type, types=types)
 
 
@@ -293,6 +299,69 @@ def generate_enum_validator(enum, **kwargs):
     return functools.partial(validate_enum, options=enum)
 
 
+@skip_if_empty
+def validate_allof_anyof(value, sub_schemas, context, method, **kwargs):
+    from flex.validation.schema import (
+        construct_schema_validators,
+    )
+
+    messages = []
+    success = []
+    for schema in sub_schemas:
+        schema_validators = construct_schema_validators(schema, context)
+        try:
+            schema_validators.validate_object(value, context=context)
+        except ValidationError as err:
+            messages.append(err.messages)
+            success.append(False)
+        else:
+            success.append(True)
+
+    if not method(success):
+        raise ValidationError(messages)
+
+    return value
+
+
+def generate_allof_validator(allOf, context, **kwargs):
+    context.setdefault('resolved_refs', [])
+    unresolved_refs = []
+    for ref in allOf:
+        if isinstance(ref, dict) and '$ref' in ref:
+            schema = ref['$ref']
+            if schema not in context['resolved_refs']:
+                unresolved_refs.append(ref)
+                context['resolved_refs'].append(schema)
+        else:
+            unresolved_refs.append(ref)
+
+    return functools.partial(validate_allof_anyof, sub_schemas=unresolved_refs,
+                             context=context, method=all)
+
+
+def generate_anyof_validator(anyOf, context, **kwargs):
+    return functools.partial(validate_allof_anyof, sub_schemas=anyOf, context=context, method=any)
+
+
+def add_polymorphism_requirements(obj, schema, context, schema_validators):
+    try:
+        object_type = obj[schema['discriminator']]
+    except KeyError:
+        raise ValidationError("No discriminator found on instance [{0}].".format(obj))
+
+    try:
+        object_schema = context['definitions'][object_type]
+    except KeyError:
+        raise ValidationError("No definition for class [{0}]".format(object_type))
+
+    from flex.validation.schema import (
+        construct_schema_validators,
+    )
+    subtype_validators = construct_schema_validators(object_schema, context)
+    schema_validators.update(subtype_validators)
+    return schema_validators
+
+
 def validate_object(obj, field_validators=None, non_field_validators=None,
                     schema=None, context=None):
     """
@@ -312,18 +381,18 @@ def validate_object(obj, field_validators=None, non_field_validators=None,
         construct_schema_validators,
     )
     schema_validators = construct_schema_validators(schema, context)
-
     if '$ref' in schema_validators and hasattr(schema_validators['$ref'], 'validators'):
         ref_ = field_validators.pop('$ref')
         for k, v in ref_.validators.items():
             if k not in schema_validators:
                 schema_validators.add_validator(k, v)
 
-    schema_validators.update(field_validators)
+    if 'discriminator' in schema:
+        schema_validators = add_polymorphism_requirements(obj, schema, context, schema_validators)
 
+    schema_validators.update(field_validators)
     schema_validators.validate_object(obj, context=context)
     non_field_validators.validate_object(obj, context=context)
-
     return obj
 
 
