@@ -1,9 +1,9 @@
 import functools
-import operator
 
 from flex.datastructures import ValidationDict
 from flex.exceptions import ValidationError
 from flex.utils import chain_reduce_partial
+from flex.functional import attrgetter, methodcaller
 from flex.context_managers import ErrorCollection
 from flex.validation.common import (
     validate_object,
@@ -40,7 +40,8 @@ def validate_status_code_to_response_definition(response, operation_definition):
     If so, return the response definition that corresponds to the status code.
     """
     status_code = response.status_code
-    operation_responses = operation_definition['responses']
+    operation_responses = {str(code): val for code, val
+                           in operation_definition['responses'].items()}
 
     key = status_code
     if key not in operation_responses:
@@ -51,7 +52,7 @@ def validate_status_code_to_response_definition(response, operation_definition):
     except KeyError:
         raise ValidationError(
             MESSAGES['response']['invalid_status_code'].format(
-                status_code, operation_responses.keys(),
+                status_code, ', '.join(operation_responses.keys()),
             ),
         )
     return response_definition
@@ -59,7 +60,7 @@ def validate_status_code_to_response_definition(response, operation_definition):
 
 def generate_response_body_validator(schema, context, **kwargs):
     return chain_reduce_partial(
-        operator.attrgetter('data'),
+        attrgetter('data'),
         functools.partial(
             validate_object,
             schema=schema,
@@ -87,12 +88,12 @@ def generate_response_header_validator(headers, context, **kwargs):
         # `response.headers.get(header_name, EMPTY)` and then feed that into
         # the type casting function and then into the validation function.
         validators.add_validator(key, chain_reduce_partial(
-            operator.methodcaller('get', key, EMPTY),
+            methodcaller('get', key, EMPTY),
             header_processor,
             header_validator,
         ))
     return chain_reduce_partial(
-        operator.attrgetter('headers'),
+        attrgetter('headers'),
         functools.partial(validate_object, field_validators=validators),
     )
 
@@ -136,7 +137,7 @@ def generate_path_validator(api_path, path_definition, parameters,
     # PATH
     in_path_parameters = filter_parameters(all_parameters, in_=PATH)
     return chain_reduce_partial(
-        operator.attrgetter('path'),
+        attrgetter('path'),
         generate_path_parameters_validator(api_path, in_path_parameters, context),
     )
 
